package ru.octol1ttle.knockdowns.client.network;

import net.minecraft.client.Minecraft;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraftforge.fml.common.network.simpleimpl.IMessage;
import net.minecraftforge.fml.common.network.simpleimpl.IMessageHandler;
import net.minecraftforge.fml.common.network.simpleimpl.MessageContext;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import ru.octol1ttle.knockdowns.client.communication.CalloutManager;
import ru.octol1ttle.knockdowns.client.communication.KnockedNotificationManager;
import ru.octol1ttle.knockdowns.client.util.DirectionalCallSound;
import ru.octol1ttle.knockdowns.common.ReviverTracker;
import ru.octol1ttle.knockdowns.common.data.IKnockdownsPlayerData;
import ru.octol1ttle.knockdowns.common.network.packets.s2c.PlayerCalloutS2CPacket;
import ru.octol1ttle.knockdowns.common.network.packets.s2c.PlayerKnockedDownS2CPacket;
import ru.octol1ttle.knockdowns.common.network.packets.s2c.SynchronizePlayerDataS2CPacket;
import ru.octol1ttle.knockdowns.common.network.packets.s2c.SynchronizeReviversS2CPacket;
import ru.octol1ttle.knockdowns.common.registry.KnockdownsSoundEvents;

import static ru.octol1ttle.knockdowns.common.KnockdownsUtils.INITIAL_REVIVE_TIME_LEFT;

public class KnockdownsClientPacketHandler {
    public static class Callout implements IMessageHandler<PlayerCalloutS2CPacket, IMessage> {
        private static final Minecraft client = Minecraft.getMinecraft();

        @SideOnly(Side.CLIENT)
        @Override
        public IMessage onMessage(PlayerCalloutS2CPacket message, MessageContext ctx) {
            client.addScheduledTask(() -> {
                if (CalloutManager.addOrUpdateCallout(message)) {
                    CalloutManager.playCalloutSound(message);
                }
            });
            return null;
        }
    }

    public static class PlayerKnockedDown implements IMessageHandler<PlayerKnockedDownS2CPacket, IMessage> {
        private static final Minecraft client = Minecraft.getMinecraft();

        @SideOnly(Side.CLIENT)
        @Override
        public IMessage onMessage(PlayerKnockedDownS2CPacket message, MessageContext ctx) {
            client.addScheduledTask(() -> {
                EntityPlayer entity = (EntityPlayer) client.world.getEntityByID(message.playerId);
                if (entity != null) {
                    IKnockdownsPlayerData data = IKnockdownsPlayerData.get(entity);
                    data.setKnockedDown(true);
                    data.setReviveTimeLeft(INITIAL_REVIVE_TIME_LEFT);
                }

                if (client.player.dimension == message.dimensionId) {
                    client.getSoundHandler().playSound(new DirectionalCallSound(KnockdownsSoundEvents.KNOCKED_DOWN, entity, message.position));
                    KnockedNotificationManager.addKnockedNotification(message.playerId, message.position);
                }
            });
            return null;
        }
    }

    public static class SynchronizePlayerData {
        private static final Minecraft client = Minecraft.getMinecraft();

        public static class KnockedDown implements IMessageHandler<SynchronizePlayerDataS2CPacket.KnockedDown, IMessage> {
            @SideOnly(Side.CLIENT)
            @Override
            public IMessage onMessage(SynchronizePlayerDataS2CPacket.KnockedDown message, MessageContext ctx) {
                client.addScheduledTask(() -> {
                    EntityPlayer entity = (EntityPlayer) client.world.getEntityByID(message.playerId);
                    if (entity != null) {
                        IKnockdownsPlayerData data = IKnockdownsPlayerData.get(entity);
                        data.setKnockedDown(message.knockedDown);
                    }
                });
                return null;
            }
        }

        public static class ReviveTimeLeft implements IMessageHandler<SynchronizePlayerDataS2CPacket.ReviveTimeLeft, IMessage> {
            @SideOnly(Side.CLIENT)
            @Override
            public IMessage onMessage(SynchronizePlayerDataS2CPacket.ReviveTimeLeft message, MessageContext ctx) {
                client.addScheduledTask(() -> {
                    EntityPlayer entity = (EntityPlayer) client.world.getEntityByID(message.playerId);
                    if (entity != null) {
                        IKnockdownsPlayerData data = IKnockdownsPlayerData.get(entity);
                        data.setReviveTimeLeft(message.reviveTimeLeft);
                    }
                });
                return null;
            }
        }

        public static class Full implements IMessageHandler<SynchronizePlayerDataS2CPacket.Full, IMessage> {
            @SideOnly(Side.CLIENT)
            @Override
            public IMessage onMessage(SynchronizePlayerDataS2CPacket.Full message, MessageContext ctx) {
                client.addScheduledTask(() -> {
                    EntityPlayer entity = (EntityPlayer) client.world.getEntityByID(message.playerId);
                    if (entity != null) {
                        IKnockdownsPlayerData data = IKnockdownsPlayerData.get(entity);
                        data.setKnockedDown(message.knockedDown);
                        data.setReviveTimeLeft(message.reviveTimeLeft);
                    }
                });
                return null;
            }
        }
    }

    public static class SynchronizeRevivers {
        private static final Minecraft client = Minecraft.getMinecraft();

        public static class Add implements IMessageHandler<SynchronizeReviversS2CPacket.Add, IMessage> {
            @SideOnly(Side.CLIENT)
            @Override
            public IMessage onMessage(SynchronizeReviversS2CPacket.Add message, MessageContext ctx) {
                client.addScheduledTask(() -> {
                    EntityPlayer knocked = (EntityPlayer) client.world.getEntityByID(message.knockedId);
                    EntityPlayer reviver = (EntityPlayer) client.world.getEntityByID(message.reviverId);
                    if (knocked != null && reviver != null) {
                        ReviverTracker.startReviving(knocked, reviver);
                    }
                });
                return null;
            }
        }

        public static class Remove implements IMessageHandler<SynchronizeReviversS2CPacket.Remove, IMessage> {
            @SideOnly(Side.CLIENT)
            @Override
            public IMessage onMessage(SynchronizeReviversS2CPacket.Remove message, MessageContext ctx) {
                client.addScheduledTask(() -> {
                    EntityPlayer knocked = (EntityPlayer) client.world.getEntityByID(message.knockedId);
                    EntityPlayer reviver = (EntityPlayer) client.world.getEntityByID(message.reviverId);
                    if (knocked != null && reviver != null) {
                        ReviverTracker.stopReviving(knocked, reviver);
                    }
                });
                return null;
            }
        }
    }
}
