package ru.octol1ttle.knockdowns.common;

import java.util.List;
import net.minecraft.client.resources.I18n;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.MobEffects;
import net.minecraft.potion.PotionEffect;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.DamageSource;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.math.MathHelper;
import net.minecraft.util.text.TextComponentTranslation;
import net.minecraftforge.event.AttachCapabilitiesEvent;
import net.minecraftforge.event.RegistryEvent;
import net.minecraftforge.event.entity.living.LivingDeathEvent;
import net.minecraftforge.event.entity.living.LivingSetAttackTargetEvent;
import net.minecraftforge.event.entity.player.AttackEntityEvent;
import net.minecraftforge.event.entity.player.PlayerEvent;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.PlayerEvent.PlayerLoggedInEvent;
import net.minecraftforge.fml.common.gameevent.PlayerEvent.PlayerLoggedOutEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent;
import ru.octol1ttle.knockdowns.Tags;
import ru.octol1ttle.knockdowns.common.data.IKnockdownsPlayerData;
import ru.octol1ttle.knockdowns.common.data.KnockdownsCapability;
import ru.octol1ttle.knockdowns.common.network.KnockdownsNetwork;
import ru.octol1ttle.knockdowns.common.network.packets.s2c.PlayerKnockedDownS2CPacket;
import ru.octol1ttle.knockdowns.common.network.packets.s2c.SynchronizePlayerDataS2CPacket;
import ru.octol1ttle.knockdowns.common.network.packets.s2c.SynchronizeReviversS2CPacket;
import ru.octol1ttle.knockdowns.common.registry.KnockdownsSoundEvents;

import static ru.octol1ttle.knockdowns.common.KnockdownsUtils.INITIAL_REVIVE_TIME_LEFT;
import static ru.octol1ttle.knockdowns.common.KnockdownsUtils.KNOCKED_HURT_PERIOD;
import static ru.octol1ttle.knockdowns.common.KnockdownsUtils.KNOCKED_INVULNERABILITY_TICKS;
import static ru.octol1ttle.knockdowns.common.KnockdownsUtils.KNOCKED_TENACITY;
import static ru.octol1ttle.knockdowns.common.KnockdownsUtils.allPlayersKnocked;
import static ru.octol1ttle.knockdowns.common.KnockdownsUtils.resetKnockedState;

@Mod.EventBusSubscriber(modid = Tags.MOD_ID)
public class KnockdownsCommonEventListener {
    public static void onFMLInit(FMLInitializationEvent event) {
        KnockdownsNetwork.registerPackets();
        KnockdownsCapability.register();
    }

    @SubscribeEvent
    public static void onSoundsRegister(RegistryEvent.Register<SoundEvent> event) {
        event.getRegistry().register(KnockdownsSoundEvents.CALLOUT);
    }

    @SubscribeEvent
    public static void onCapabilitiesAttach(AttachCapabilitiesEvent<Entity> event) {
        if (event.getObject() instanceof EntityPlayer) {
            event.addCapability(KnockdownsCapability.ID, new KnockdownsCapability());
        }
    }

    @SubscribeEvent
    public static void onPlayerTick(TickEvent.PlayerTickEvent event) {
        MinecraftServer server = event.player.getServer();
        if (event.phase == TickEvent.Phase.START || server == null) {
            return;
        }
        EntityPlayerMP player = (EntityPlayerMP) event.player;
        IKnockdownsPlayerData data = IKnockdownsPlayerData.get(player);
        if (!data.isKnockedDown()) {
            return;
        }

        if (allPlayersKnocked(server, player)) {
            player.attackEntityFrom(DamageSource.GENERIC, player.getMaxHealth());
            return;
        }

        List<EntityPlayer> revivers = ReviverTracker.getRevivers(player);
        if (!revivers.isEmpty()) {
            data.setReviveTimeLeft(data.getReviveTimeLeft() - revivers.size());
            KnockdownsNetwork.sendToMultiple(
                    new SynchronizePlayerDataS2CPacket.ReviveTimeLeft(player.getEntityId(), data.getReviveTimeLeft()),
                    revivers,
                    player
            );

            if (data.getReviveTimeLeft() <= 0) {
                resetKnockedState(player, data);

                player.setEntityInvulnerable(false);
                player.setHealth(player.getMaxHealth() * 0.3f);
                player.setAbsorptionAmount(0.0f);
            }
            return;
        }

        int oldReviveTimeLeft = data.getReviveTimeLeft();
        data.setReviveTimeLeft(Math.min(INITIAL_REVIVE_TIME_LEFT, oldReviveTimeLeft + 1));
        if (data.getReviveTimeLeft() != oldReviveTimeLeft) {
            KnockdownsNetwork.sendToPlayer(
                    new SynchronizePlayerDataS2CPacket.ReviveTimeLeft(player.getEntityId(), data.getReviveTimeLeft()),
                    player
            );
        }

        data.setTicksKnocked(data.getTicksKnocked() + 1);

        int period = MathHelper.floor(KNOCKED_HURT_PERIOD * 20);
        if (data.getTicksKnocked() >= KNOCKED_INVULNERABILITY_TICKS && data.getTicksKnocked() % period == 0) {
            player.setEntityInvulnerable(false);
            player.attackEntityFrom(DamageSource.GENERIC, player.getMaxHealth() / (KNOCKED_TENACITY / KNOCKED_HURT_PERIOD));
        }
    }

    @SubscribeEvent
    public static void onPlayerDeath(LivingDeathEvent event) {
        if (!(event.getEntityLiving() instanceof EntityPlayerMP)) {
            return;
        }

        EntityPlayerMP player = (EntityPlayerMP) event.getEntityLiving();
        IKnockdownsPlayerData data = player.getCapability(KnockdownsCapability.CAPABILITY, null);
        if (data == null) {
            return;
        }

        if (data.isKnockedDown() || allPlayersKnocked(player.getServer(), player)) {
            ReviverTracker.clearRevivers(player);
            return;
        }

        player.clearActivePotions();
        player.setEntityInvulnerable(true);
        player.setHealth(1.0f);
        player.setAbsorptionAmount(player.getMaxHealth() - 1.0f);
        player.extinguish();
        player.setAir(300);
        player.clearElytraFlying();

        player.addPotionEffect(new PotionEffect(MobEffects.SLOWNESS, 6000, 3));

        Entity trueSource = event.getSource().getTrueSource();
        if (trueSource instanceof EntityLiving) {
            ((EntityLiving) trueSource).setAttackTarget(null);
        }

        data.setKnockedDown(true);
        data.setReviveTimeLeft(INITIAL_REVIVE_TIME_LEFT);
        data.setTicksKnocked(0);

        KnockdownsNetwork.sendToAll(new PlayerKnockedDownS2CPacket(player.getEntityId(), player.dimension, player.getPositionEyes(1).add(0, 1, 0)));

        TextComponentTranslation deathMessage = (TextComponentTranslation) player.getCombatTracker().getDeathMessage();

        String knockdownKey = deathMessage.getKey().replace("death.", "knockdown.");
        player.getServer().getPlayerList().sendMessage(
                I18n.hasKey(knockdownKey)
                        ? new TextComponentTranslation(knockdownKey, deathMessage.getFormatArgs())
                        : deathMessage,
                true
        );

        event.setCanceled(true);
    }

    @SubscribeEvent
    public static void onMobTarget(LivingSetAttackTargetEvent event) {
        if (event.getTarget() instanceof EntityPlayer && IKnockdownsPlayerData.get((EntityPlayer) event.getTarget()).isKnockedDown()) {
            ((EntityLiving)event.getEntityLiving()).setAttackTarget(null);
        }
    }

    @SubscribeEvent
    public static void onPlayerStartTracking(PlayerEvent.StartTracking event) {
        if (event.getTarget() instanceof EntityPlayerMP) {
            IKnockdownsPlayerData data = IKnockdownsPlayerData.get((EntityPlayer) event.getTarget());
            if (data.isKnockedDown()) {
                KnockdownsNetwork.sendToPlayer(
                        new SynchronizePlayerDataS2CPacket.KnockedDown(
                                event.getTarget().getEntityId(),
                                data.isKnockedDown()
                        ),
                        (EntityPlayerMP) event.getEntityPlayer()
                );
            }
        }
    }

    @SubscribeEvent
    public static void onKnockedAttack(AttackEntityEvent event) {
        if (IKnockdownsPlayerData.get(event.getEntityPlayer()).isKnockedDown()) {
            event.setCanceled(true);
        }
    }

    @SubscribeEvent
    public static void onKnockedInteraction(PlayerInteractEvent event) {
        if (IKnockdownsPlayerData.get(event.getEntityPlayer()).isKnockedDown()) {
            if (!(event instanceof PlayerInteractEvent.RightClickBlock) && event.isCancelable()) {
                event.setCanceled(true);
                event.setCancellationResult(EnumActionResult.FAIL);
            }
        } else if (event instanceof PlayerInteractEvent.EntityInteract) {
            onPlayerInteraction((PlayerInteractEvent.EntityInteract) event);
        }
    }

    public static void onPlayerInteraction(PlayerInteractEvent.EntityInteract event) {
        if (event.getEntityLiving() instanceof EntityPlayerMP) {
            EntityPlayerMP knocked = (EntityPlayerMP) event.getEntityLiving();
            if (IKnockdownsPlayerData.get(knocked).isKnockedDown()) {
                KnockdownsNetwork.sendToMultiple(
                        new SynchronizeReviversS2CPacket.Add(event.getEntityLiving().getEntityId(), event.getEntityPlayer().getEntityId()),
                        ReviverTracker.getRevivers(knocked),
                        knocked
                );
                ReviverTracker.startReviving(knocked, event.getEntityPlayer());
            }
        }
    }

    @SubscribeEvent
    public static void onPlayerJoin(PlayerLoggedInEvent event) {
        IKnockdownsPlayerData data = IKnockdownsPlayerData.get(event.player);
        KnockdownsNetwork.sendToPlayer(
                new SynchronizePlayerDataS2CPacket.Full(event.player.getEntityId(), data.isKnockedDown(), data.getReviveTimeLeft()),
                (EntityPlayerMP) event.player
        );
    }

    @SubscribeEvent
    public static void onPlayerLeave(PlayerLoggedOutEvent event) {
        ReviverTracker.clearRevivers(event.player);
    }
}
